<?php
/**
 * Cleantalk joomla plugin
 * 
 * @version 1.4.1
 * @package Cleantalk
 * @subpackage Joomla
 * @author Сleantalk team (shagimuratov@cleantalk.ru)
 * @copyright (C) 2012 Сleantalk team (http://cleantalk.ru)
 * @license GNU/GPL: http://www.gnu.org/copyleft/gpl.html
 *
 */

defined('_JEXEC') or die('Restricted access');
jimport('joomla.plugin.plugin');

class plgSystemCleantalk extends JPlugin {
	/**
	* Plugin version string for server
	*/
        const ENGINE = 'joomla-130';
    
	/**
	* Cleantalk instance
	*/
	static $CT;

	/**
	* Stoplist checking flag
	*/
	static $stoplist_check;

	/**
	* Allow links flag
	*/
	static $allow_links;

	/**
	* Tables exist and ready flag
	* If set then tables exist and filled by initial data
	*/
	static $tables_ready = FALSE;

	/**
	* Constructor
	* @param $subject
	* @param $config
	* @return void
	*/
	function plgSystemCleantalk(&$subject, $config) {
	    parent::__construct($subject, $config);
	}

/* List of available triggers in JComments 2.3.0 - jcomments.ajax.php

onJCommentsCaptchaVerify
onJCommentsCommentBeforeAdd	- used, working
onJCommentsCommentAfterAdd
onJCommentsCommentBeforeDelete
onJCommentsCommentAfterDelete	- used, but not called from comments admin panel
onJCommentsCommentBeforePublish - used, working
onJCommentsCommentAfterPublish
onJCommentsCommentBeforeChange
onJCommentsCommentAfterChange
onJCommentsCommentBeforeVote
onJCommentsCommentAfterVote
onJCommentsCommentBeforeReport
onJCommentsCommentAfterReport
onJCommentsUserBeforeBan
onJCommentsUserAfterBan

*/

	/**
        * onJCommentsCommentBeforeAdd trigger
        * @access public
        * @param JCommentsDB $comment
        * @return bolean true
        * @since 1.5
        */
        function onJCommentsCommentBeforeAdd(&$comment)	{
            $plugin_groups = array();
    	    $param_groups = $this->params->get('groups');
            if(is_array($param_groups)){
                foreach ($param_groups as $group){
                    array_push($plugin_groups, (int)$group);
                }
            }else{
                array_push($plugin_groups, (int)$param_groups);
            }

    	    $user = JFactory::getUser();
    	    if(method_exists($user, 'getAuthorisedGroups')){    // 1.6+
    		$user_groups = $user->getAuthorisedGroups();
    	    }else{                                              // 1.5
    		$user_groups = array();
                if($user->guest){
                    array_push($user_groups, 29);
                }else{
                    array_push($user_groups, $user->gid);
                }
    	    }
    	    foreach ($user_groups as $group){
    		if(in_array($group, $plugin_groups)){
		    switch($comment->object_group){
			case 'com_content':
				$article = & JTable::getInstance('content');
				$article->load($comment->object_id);
				$baseText = $article->introtext . '<br>' . $article->fulltext;
			break;
			default:
				$baseText = '';
		    }

		    $db = & JCommentsFactory::getDBO();
		    $query = "SELECT comment "
				. "\nFROM #__jcomments "
				. "\nWHERE published = 1 "
				. "\n  AND object_group = '" . $db->getEscaped($comment->object_group) . "'"
				. "\n  AND object_id = " . $comment->object_id
				. (JCommentsMultilingual::isEnabled() ? "\nAND lang = '" . JCommentsMultilingual::getLanguage() . "'" : "")
				. " ORDER BY id DESC "
				. " LIMIT 10 "
				;
		    $db->setQuery($query);
		    $prevComments = $db->loadResultArray();
		    $prevComments = $prevComments == NULL ? '' : implode("\n\n", $prevComments);

		    $ctResponse = self::ctSendRequest(
				'check_message', 
				array(
				    'base_text'=> ($baseText . "\n\n\n\n" . $prevComments),
				    'user_name'=>$comment->name,
				    'user_email'=>$comment->email,
				    'message'=>$comment->comment,
				    'url'=>'',
				    'session_ip'=>$comment->ip
				) 
		    );
		    
		    if(!empty($ctResponse) && is_array($ctResponse)){
			if($ctResponse['blacklisted'] > 0){
				JCommentsAJAX::showErrorMessage($ctResponse['comment'], 'comment');
				return false;
			}else if($ctResponse['allow'] == 0){
				$comment->published = false;
				$comment->comment = self::$CT->addCleantalkComment($comment->comment, $ctResponse['comment']);
			}
		    }
		    return true;
    		} //if(in_array($group, $plugin_groups))
    	    } //foreach
        }

	/**
        * onJCommentsCommentBeforePublish trigger
        * @access public
        * @param JCommentsDB $comment
        * @return bolean true
        * @since 1.5
        */
	function onJCommentsCommentBeforePublish(&$comment){
	    self::moderateMessage($comment->comment, 1);
	    return true;
	}

	/**
        * onJCommentsCommentAfterDelete trigger
        * @access public
        * @param JCommentsDB $comment
        * @return bolean true
        * @since 1.5
        */
	function onJCommentsCommentAfterDelete(&$comment){
	    self::moderateMessage($comment->comment, 0);
	    return true;
	}

	/**
	 * Function to send the results of moderation
	 * @param $message
	 * @param $allow
	 * @return void
	 */
	function moderateMessage(&$message, $allow){
	    self::getCleantalk();
	    $hash = self::$CT->getCleantalkCommentHash($message);
	    $resultMessage = self::$CT->delCleantalkComment($message);

	    if($hash != NULL){
	    	$ctFbParams = array(
	    		'moderate'=>array(
				array('msg_hash'=>$hash, 'is_allow'=>$allow),
			),
		);

		self::ctSendRequest(
		    'send_feedback', 
		    $ctFbParams
		);
	    }

	    $message = $resultMessage;
	}

	/**
	* Interface to XML RPC server
	* $mehod - method name
	* $params - array of XML params
	* return XML RPS server response
	*/
	function ctSendRequest($method, $params){
	    self::getCleantalk();
	    
	    switch($method){
		case 'check_message':
			$sender_id = self::$CT->getSenderId();
			$params = array(
				$params['message'],
				$params['base_text'],
				self::$CT->config['auth_key'],
				self::ENGINE,
				$params['url'],
				self::$stoplist_check,
				self::$CT->config['response_lang'],
				$params['session_ip'],
				$params['user_email'],
				$params['user_name'],
				$sender_id,
				self::$allow_links
			);
			break;
		case 'send_feedback':
			$feedback = array();
			foreach($params['moderate'] as $msgFeedback)
				$feedback[] = $msgFeedback['msg_hash'] . ':' . intval($msgFeedback['is_allow']);
			
			$feedback = implode(';', $feedback);
				
			$params = array(
				self::$CT->config['auth_key'],
				$feedback,
			);
			break;
		default: 
			return NULL;
	    }

	    $plugin_url = self::$CT->config['server_url'];
	    $config = self::dbGetServer();
	    $result = NULL;

	    if ((isset($config['ct_work_url']) && $config['ct_work_url'] !== '') && ($config['ct_server_changed'] + $config['ct_server_ttl'] > time())) {
		$result = self::$CT->xmlRequest2(
			$method,
			$params,
			$config['ct_work_url']
		);
	    }

	    if (!isset($result) || $result->faultCode()) {
                $matches = array();
		preg_match("#^(http://|https://)([a-z\.\-0-9]+):?(\d*)$#i", $plugin_url, $matches);
		$url_prefix = $matches[1];
		$pool = $matches[2];
		$port = $matches[3];
		if(empty($url_prefix))
		    $url_prefix = 'http://';
		if(empty($pool)){
            	    $result = array(
                            'allow'=>0,
                            'comment'=>'Can\'t connect to cleantalk.ru',
                            'blacklisted'=>0,
            		);
		}else{
		    foreach (dns_get_record($pool, DNS_A) as $server){
			$server_host = gethostbyaddr($server['ip']);
			$work_url = $url_prefix . $server_host;
			if ($server['host'] === 'localhost')
				$work_url = $url_prefix . $server['host'];
			
			$work_url = ($port !== '') ? $work_url . ':' . $port : $work_url;

			$result = self::$CT->xmlRequest2(
				$method,
				$params,
				$work_url	
			);

			if (!$result->faultCode()){
				self::dbSetServer($work_url, $server['ttl'], time());
				break;
			}
		    }
		    if (!$result){
			$result = self::$CT->xmlRequest2(
				$method,
				$params,
				$plugin_url
			);
		    }	
		}
	    }
	
	    if(is_object($result) && !$result->faultCode()){
		$result = $result->value();
		
		if($method === 'check_message' && $sender_id == '' && isset($result['sender_id']))
			self::$CT->setSenderId($result['sender_id']);
	    }else{
                $result = array(
                            'allow'=>0,
                            'comment'=>'Can\'t connect to cleantalk.ru',
                            'blacklisted'=>0,
            		);
	    }

	    return $result;
	}

	/**
	* Cleantalk instance
	* @return Cleantalk instance
	*/
	function getCleantalk() {
	    if(!isset(self::$CT)){
		require_once(dirname(__FILE__).DS.'cleantalk.class.php');
		$plugin = JPluginHelper::getPlugin('system', 'cleantalk');

    		if(class_exists('JParameter')){			//1.5
                    $jparam = new JParameter( $plugin->params );
                    $p_key = $jparam->def('apikey','');
                    $p_url = $jparam->def('server','');
                    $p_lang = $jparam->def('lang','');
                    $p_stoplist = $jparam->def('stoplist', '1');
                    $p_links = $jparam->def('links','0');
        	}else{						//1.6+
                    $jreg = new JRegistry($plugin->params);
                    $p_key = $jreg->get('apikey','');
                    $p_url = $jreg->get('server','');
                    $p_lang = $jreg->get('lang','');
                    $p_stoplist = $jreg->get('stoplist','1');
                    $p_links = $jreg->get('links','0');
        	}

        	if(empty($p_lang)){
	    	    $lang = & JFactory::getLanguage();
	    	    $p_lang = substr($lang->getTag(), 0, 2);
		}

		self::$stoplist_check = $p_stoplist;
		self::$allow_links = $p_links;
		self::$CT = new Cleantalk(array(
						'auth_key' => $p_key,
						'response_lang' => $p_lang,
						'server_url' => $p_url
						));
	    }
            return self::$CT;
	}
	
	/**
	* Cleantalk tables creator
	* @return bool
	*/
	function initTables() {
	    $db = &JFactory::getDBO();
	    $prefix = $db->getPrefix();
	    $arrTables = $db->getTableList();
 
	    if (!empty($arrTables)) {
		if (!in_array($prefix.'ct_curr_server', $arrTables)) {
		    $db->setQuery("CREATE TABLE `#__ct_curr_server` (
			`id` int(11) unsigned NOT NULL auto_increment,
			`ct_work_url` varchar(100) default NULL,
			`ct_server_ttl` int(11) NOT NULL default '0',
			`ct_server_changed` int(11) NOT NULL default '0',
			PRIMARY KEY (`id`)
			) ENGINE=MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8");
		    $db->query();
		}
	    
		$db->setQuery ("SELECT count(*) FROM #__ct_curr_server");
		$row = $db->loadRow();
		if($row[0] == 0){
		    $db->setQuery (
		    "INSERT  ".
		    "INTO #__ct_curr_server (ct_work_url,ct_server_ttl,ct_server_changed ) ".
		    "VALUES ('', 0, 0)");
		    if($db->query() !== FALSE) 
			self::$tables_ready = TRUE;
		}else{
		    self::$tables_ready = TRUE;
		}
	    }
	    return self::$tables_ready;
	}
	
	/**
	* Current server getter
	* @return array
	*/
	function dbGetServer() {
	    if(!self::$tables_ready){
		self::initTables();
	    }
	    $db = &JFactory::getDBO();
	    $db->setQuery ("SELECT ct_work_url,ct_server_ttl,ct_server_changed FROM #__ct_curr_server ORDER BY id LIMIT 1");
	    $row = $db->loadAssoc();
	    return $row;
	}

	/**
	* Current server setter
	$ct_work_url
	$ct_server_ttl
	$ct_server_changed
	* @return null
	*/
	function dbSetServer($ct_work_url,$ct_server_ttl,$ct_server_changed) {
	    if(!self::$tables_ready){
		self::initTables();
	    }
	    $db = &JFactory::getDBO();
	    $db->setQuery (
	    "UPDATE #__ct_curr_server " .
	    " SET " .
	    "ct_work_url = '" . $ct_work_url . "', " .
	    "ct_server_ttl = " . $ct_server_ttl . ", " .
	    "ct_server_changed = " . $ct_server_changed);
	    $db->query();
	}
}
